# HOBJPROC -- Correct for fiber throughput differences
# By Doug Mink, SAO/TDC
# February 15, 2007

procedure hobjproc (images)

string	images = ""	{prompt="Input multispec images to correct"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool saveold = yes	{prompt="Save unprocessed files to Notran/ (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Log each aperture (y or n)"}

begin
	bool vb, db, save, del
	file imin, imout, templist, imvar
	string ims, tchar

	if (!deftask ("emsao")) {
	    rvsao (motd=no)
	    }
	if (!deftask ("getcol")) {
	    wcstools (motd=no)
	    }

	ims = images
	vb = verbose
	db = debug
	del = deltemp
	save = saveold

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("htranlist")
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Throughput correction
	    if (imcorr != "") {
		htran1 (imin, deltemp=del, saveold=save, verbose=vb, debug=db)

# Sky absorption removal
	    if (imsky != "") {
		habsky1 (imin, deltemp=del, saveold=save, verbose=vb, debug=db)
		}

# Fit and remove red leak
	    hredleak1 (imin, deltemp=del, saveold=save, verbose=vb, debug=db)

# Split into one object per file
	    hsplit (imin, deltemp=del, saveold=save, verbose=vb, debug=db)

# Split variance file, if it exists, into one object per file
	    imroot (imin)
	    imvar = imroot.root // "_v.ms.fits"
	    if (access (imvar))
		hsplit (imvar, deltemp=del, saveold=save, verbose=vb, debug=db)
	    }

	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
end

# Jun 10 2004	New task
# Sep  2 2004	Handle multiple files in one run
# Sep 23 2004	Change "transmission" to "throughput"
# Oct 18 2004	Change FIBRTRAN keyword to HTRAN
# Oct 18 2004	Print value of HTRAN/FIBRTRAN if already processed
# Oct 18 2004	Split off geometric correction to HGEOM task

# Jan 10 2005	Improve input filename options
# Jan 13 2005	Fix typo in temporary list variable name on deletion
# Mar  9 2005	Call HGEOM1 to do geometric correction, not HGEOM
# Mar  9 2005	Move computation to single image HTRAN1
# Mar 17 2005	Do not turn off deltemp if reading from file list
# May 31 2005	Add HABSKY1 to remove sky absorption
# Jun  9 2005	Add HREDLEAK1 to remove red leak from spectra
# Aug  8 2005	Drop out of program if throughput correction file is not given
# Aug  8 2005	Drop out of program if throughput correction file cannot be read
# Aug  8 2005	Skip sky correction if sky absorption file is not given
# Aug  8 2005	Drop out of program if sky absorption file cannot be read
# Aug 10 2005	Load wcstools and rvsao only if they are not already loaded

# Aug 23 2006	Drop geometric connection; it should only be used for sky

# Feb 14 2007	Rename from htran to hobproc; drop geometry correction
# Feb 15 2007	Pass only debugging parameters to subtasks
# Feb 26 2007	Add variance splitting
