# HPROC1 - Run image processing tasks one amp at a time, then
          combine 4 amps on 2 chips into 1 Hectospec or 2 Hectochelle images
# December 14, 2004
# By Doug Mink, SAO Telescope Data Center

procedure procamps (images)

string	images=""	{prompt="Images to merge"}
bool	hectospect=yes	{prompt="yes for Hectospec, no for Hectochelle"}
bool	fixpreamp=yes	{prompt="Run preampfix to clean overscan (y or n)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, temp4, im2, im4, imin, templist, imout
file	imfile, newfile, imex0
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar
string	amp1, amp2, amp3, amp4
bool	dellist, vb, procdone

	vb = verbose
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("HPROC1: Run image processing tasks one amp at a time, then combine amps")
	    print ("Usage: hproc <input images with 4 extensions>")
	    return
	    }

# Make directory in which to save raw multi-extension files
	if (saveold) {
	    if (!access ("Raw"))
		mkdir ("Raw")

# Make directory in which to save preampfixed multi-extension files
	    if (!access ("Unproc"))
		mkdir ("Unproc")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	if (tchar == '@') {
	    templist = substr (ims,2,100)
	    deltemp = no
	    }
	else {
	    templist = mktemp ("proclist")
	    files (ims, >templist)
	    }
	list = templist
        while (fscan (list,imin) != EOF) {

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[1]"
	    imgetp (imsec, "HPROC")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HPROC1: ", imin, " has already been processed")
		    print ("HPROC1 = ", imgetp.value)
		    }
		procdone = yes
		}
	    else {
		procdone = no
		}

# Find root name of image
	    if (!procdone) {
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		imin = imin // imx
		}
#	    if (vb) {
#		print ("HPROC: image root is ",imroot)
#		}

# Fix preamp output using Brian McLeod's task
	    temp1 = mktemp (imroot) // "p.fits"
	    if (verbose)
		print ("HPROC: Making preamp correction for ",imin)
	    preampfix (imin, temp1)
	    if (verbose)
		print ""

# Make history line including time for preamp correction
            gettime ()
	    history = gettime.date // " " // gettime.time // " preamplifier correction made"
	    imsec = temp1 // "[1]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[2]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[3]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)
	    imsec = temp1 // "[4]"
	    hedit (imsec, "PREAMPFX", history, add+, verify-)

#  Save preampfixed file to Unproc/
	    if (saveold) {
		imr = "Unproc/" // imin
		if (access (imr))
		    delete (imr)
		copy (temp1, imr)
		}

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
	    ccdproc.bpmasks = ""
	    ccdproc.ccdtype = ""
	    ccdproc.noproc = no
	    ccdproc.xtalkcor = no
	    ccdproc.fixpix = no
	    ccdproc.overscan = no
	    ccdproc.trim = no
	    ccdproc.zerocor = no
	    ccdproc.darkcor = no
	    ccdproc.flatcor = no
	    ccdproc.sflatcor = no
	    ccdproc.split = no
	    ccdproc.merge = no
	    ccdproc.xtalkfile = ""
	    ccdproc.fixfile = "BPM"
	    ccdproc.saturation = "INDEF"
	    ccdproc.sgrow = 0
	    ccdproc.bleed = "INDEF"
	    ccdproc.btrail = 20
	    ccdproc.bgrow = 0
	    ccdproc.biassec = "[1080:1120,3:4610]"
	    ccdproc.trimsec = "!TRIMSEC"
	    ccdproc.zero = "ZERO"
	    ccdproc.dark = "DARK"
	    ccdproc.flat = "FLAT"
	    ccdproc.sflat = ""
	    ccdproc.minreplace = 1.
	    ccdproc.interactive = no
	    ccdproc.function = "legendre"
	    ccdproc.order = 1
	    ccdproc.sample = "*"
	    ccdproc.naverage = 1
	    ccdproc.niterate = 1
	    ccdproc.low_reject = 3.
	    ccdproc.high_reject = 3.
	    ccdproc.grow = 0.
	    ccdproc.fd = ""
	    ccdproc.fd2 = ""
	    temp2 = mktemp (imroot) // "pc" // imx
	    if (verbose)
		print ("HPROC: fixpix, overscan, and trim ", imin)
	    mscred.ccdproc (temp1, output=temp2, fixpix+, overscan+, trim+)

#  Adjust amplifiers to same gain
	    temp3 = mktemp (imroot) // "pcg" // imx
	    if (verbose)
		print ("PROCAMPS: gain correction for ", imin)
	    gaincorr (temp2, temp3)

#  Rename original file
	    if (saveold) {
		imr = "Raw/" // imin
		if (access (imr))
		    delete (imr)
		rename (imin, imr)
		}
	    else {
		delete (imin)
		}

#  Rename result to same as original file
	    rename (temp3, imin)

	    if (deltemp) {
		imdelete (temp1)
		imdelete (temp2)
		}

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " fixpix, overscan, and trim"
	    imsec = imin // "[1]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    imsec = imin // "[2]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    imsec = imin // "[3]"
            hedit (imsec, "HPROC", history, add+, verify-)
	    imsec = imin // "[4]"
            hedit (imsec, "HPROC", history, add+, verify-)
		}

# Do not merge amplifiers twice
	    imgetp.default = "0"
	    imex0 = imin // "[0]"
	    imgetp (imex0, "HMERGE")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HPROC1: ", imin, " has already been merged")
		    print ("HPROC1 = ", imgetp.value)
		    }
		procdone = yes
		}
	    else {
		procdone = no
		}

# Find root name of image
	    if (!procdone) {
	    imroot = imin
	    smatch (imroot, imx)
	    if (smatch.found) {
		imroot = substr (imroot,1,smatch.start-1)
		}
	    else {
		imroot = imageout
		}
	    if (vb) {
		print (" HPROC: image root is ",imroot)
		}

#  Split out one image per amplifier
	    temp1 = mktemp ("hamp1") // imx
	    amp1 = imroot // "[1]"
	    imcopy (amp1, temp1)
	    temp2 = mktemp ("hamp2") // imx
	    amp2 = imroot // "[2]"
	    imcopy (amp2, temp2)
	    temp3 = mktemp ("hamp3") // imx
	    amp3 = imroot // "[3]"
	    imcopy (amp3, temp3)
	    temp4 = mktemp ("hamp4") // imx
	    amp4 = imroot // "[4]"
	    imcopy (amp4, temp4)

# Move original file to Unflat directory
	    if (saveold) {
		if (!access ("Unmerged/")) {
		    mkdir ("Unmerged")
		    }
		newfile = "Unmerged/" // imin
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		movefiles (imin, "Unmerged")
		if (vb) {
		    print ("HPROC: Original ", imin, " => Unmerged/")
		    }
		}
	    else {
		delete (imin)
		}

#  If Hectospec, join all four amps on both chips into a single image
	    if (hectospec) {
		arg1 = temp1 // "," // temp2 // "[-*,*]," // temp3 // "," // temp4 // "[-*,*]"
		imjoin (arg1, imin, 1)

#	Make history line including time for preamp correction
        	gettime ()
		history = gettime.date // " " // gettime.time // " 4 amplifiers merged into one image"
		hedit (imin, "HMERGE", history, add+, verify-)
		}
	    else {
		arg1 = temp1 // "," // temp2 // "[-*,*]"
		imout = imroot // "_1" // imx
		imjoin (arg1, imout, 1)
		arg1 = temp3 // "," // temp4 // "[-*,*]"
		imout = imroot // "_2" // imx
		imjoin (arg1, imout, 1)
		}

	    if (deltemp) {
		imdelete (temp1)
		imdelete (temp2)
		imdelete (temp3)
		imdelete (temp4)
		}
	    }
	    }

	if (deltemp)
	    delete (templist)
end

# May 20 2004	New program to process list of multiextension images
# May 21 2004	Add header line for preamp fix; move original file to Raw/
# Jun  8 2004	Change name of task from PROCAMPS to HPROC
# Jun  8 2004	Save preampfixed, unccdproc'ed files to Unproc/
# Jun 22 2004	Program to make a images from four amplifier image extensions
# Jun 23 2004	Save unprocessed files only if saveold=yes
# Sep  2 2004	Add HPROC keyword and skip file if it exists already
# Sep  2 2004	Do not process image if HMERGE keyword already exists
# Sep  2 2004	Merge multiple files
# Oct 13 2004	Write HPROC keyword to each amplifier's image extension
# Oct 13 2004	Check for HMERGE in 0th extension (only extension if processed)
# Oct 18 2004	Print HPROC keyword if it is present
# Oct 18 2004	Print HMERGE value if amplifiers are already merged
# Dec  3 2004	Add hmerge.cl to simplify pipeline
# Dec 14 2004	Merge Hectochelle to one file per chip
