# File hectospec/Scripts/hskyfind.cl
# By Doug Mink, after Bill Wyatt
# September 13, 2007

procedure hskyfind (objlist, skylist)

string	objlist=""	{prompt="File of object file names, aps, and fiber pos"}
string	skylist=""	{prompt="File of sky file names, aps, and fiber pos"}

string	skymethod="skyfit"	{prompt="Sky removal method (skyfit or skysub)"}
file	skyline="OH8399.dat"	{prompt="Sky line file to use for scaling factor"}
file	linedir="hectospec$lib/"	{prompt="Directory for sky line data"}
bool	usearea=yes		{prompt="Use sky line area vs. height in scaling"}
string	skysort="skydist"	{prompt="Closest skies in (skydist/aperture)"}
int	nskies = 6		{prompt="Maximum number of skies to coadd"}
real	fiberdist = 0.0		{prompt="Max Fiber distance to sky (0=use all)"}
string	destdir="skysub/"	{prompt="Output dir for combined sky files"}
bool	deltemp=yes		{prompt="Delete temporary files (y or n)"}
bool	verbose=yes		{prompt="note progress"}
bool	debug=no		{prompt="note progress in detail"}

begin
	bool vb, dt, db
	string vstring, comlist, skyroot, history
	string ts1,ts2,ts3,ts4,ts5
	file imsky,skydir,imobj,skycomb,olist,slist,newfile,newpath,objskydb
	file ddir
	int ap, fiber, nobj
	real fibx, fiby

	vb = verbose
	db = debug
	dt = deltemp
	olist = objlist
	slist = skylist
	ddir = destdir
	fixdir (ddir)
	ddir = fixdir.newdir

# Check for existence of input files
	if (!access (olist)) {
	    print ("HSKYFIND: Cannot read object list file ", olist)
	    return
	    }
	else if (db) {
	    print ("HSKYFIND: Reading from object list file ", olist)
	    }
	if (!access (slist)) {
	    print ("HSKYFIND: Cannot read sky list file ", slist)
	    return
	    }
	else if (db) {
	    print ("HSKYFIND: Reading from sky list file ", slist)
	    }

# Set parameters for combining sky spectra
	imcombine.rejmask=""
	imcombine.combine="average"
	imcombine.reject="avsigclip"
	imcombine.outtype="real"
	imcombine.scale="none"
	imcombine.zero="none"
	imcombine.weight="!EXPTIME"
	imcombine.mclip=yes
	imcombine.lsigma=3.
	imcombine.hsigma=3.
	imcombine.sigscale=0.1
	imcombine.logfile=""

# Set up parameters for sky matching task
	hskymatch.skylist = slist
	hskymatch.skysort = skysort
	hskymatch.maxskies = nskies
	hskymatch.maxdist = fiberdist
	hskymatch.verbose = vb
	hskymatch.debug = db
	skydir = ddir // "skycomb/"
	skyroot = skydir // "sky_"
	if (!access (skydir))
	    mkdir (skydir)
	if (db)
	    print ("HSKYFIND: Writing combined sky files to ",skydir)

# Set up table for matched sky apertures in combined sky file name
	objskydb = ddir // "objsky.db"
	if (access (objskydb))
	    delete (objskydb)

# Set up parameters for SVD sky removal task
	hskyfit1.objap = 1
	hskyfit1.objband = 1
	hskyfit1.objdir = ""
	hskyfit1.fitmax = 1.1
	hskyfit1.skyap = 1
	hskyfit1.skyband = 1
	hskyfit1.skydir = ""
	hskyfit1.outdir = ddir
	hskyfit1.plotres = yes
	hskyfit1.verbose = vb
	hskyfit1.debug = db

# Set up parameters for line-scaled subtraction sky removal task
	hskysub1.objap = 1
	hskysub1.objband = 1
	hskysub1.objdir = ""
	hskysub1.skyap = 1
	hskysub1.skyband = 1
	hskysub1.skydir = ""
	hskysub1.outdir = ddir
	hskysub1.skyline = skyline
	hskysub1.linedir = linedir
	hskysub1.usearea = usearea
	hskysub1.deltemp = dt
	hskysub1.verbose = vb
	hskysub1.debug = db

# Set up parameters for adding parameters to header
	hedit.add = yes
	hedit.addonly = no
	hedit.delete = no
	hedit.verify = no
	hedit.show = debug
	hedit.update = yes

# Loop through object spectra to have background removed
	if (db)
	    print ("HSKYFIND: About to open file ",olist)
	nobj = 0
	list = olist

# Skip first two lines which are column headers
        if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
            if (fscan (list, ts1,ts2,ts3,ts4,ts5) != EOF) {
                }
            }

# Read the rest of the object file
	while (fscan (list, imobj, ap, fiber, fibx, fiby) != EOF) {

	    if (debug)
		print ("HSKYFIND: ",imobj," ",ap, fibx, fiby)

	    if (!access (imobj)) {
		print ("HSKYFIND: Cannot read object spectrum ",imobj)
		next
		}
	    nobj = nobj + 1
	    if (nobj == 1) {
		print ("object_file	nsky	sky_file", > objskydb)
		print ("-----------	----	--------", >> objskydb)
		}

# Select sky files to combine for this object
	    hskymatch (imobj, ap, fibx, fiby)
	    if (hskymatch.nsky > 0) {
		if (verbose) {
		    print ("HSKYFIND: ",hskymatch.nsky," skies found for ",imobj)
		    }

# Combine selected sky files, if combined file does not already exist
		skycomb = skyroot // hskymatch.skyaps // ".fits"
		if (!access (skycomb)) {
		    if (vb) {
			if (skymethod == "skyfit") {
			    print ("HSKYFIND: Fitting new sky: ",skycomb)
			    }
			else {
			    print ("HSKYFIND: Subtracting new sky: ",skycomb)
			    }
			}
		    imcombine (hskymatch.skyfiles, skycomb)
		    hedit (skycomb, "NSKYMAX",nskies, add+, verify-)
		    hedit (skycomb, "NSKYFIB",hskymatch.nsky, add+, verify-)
		    hedit (skycomb, "SKYDIST",fiberdist, add+, verify-)
		    hedit (skycomb, "SKYCERT",hskyproc.skycert, add+, verify-)
		    hedit (skycomb, "SKYSTDLM",hskyproc.skystdlim, add+, verify-)
		    }
		else if (vb) {
		    if (skymethod == "skyfit") {
			    print ("HSKYFIND: Fitting existing sky: ",skycomb)
			}
		    else {
			print ("HSKYFIND: Subtracting existing sky: ",skycomb)
			}
		    }

# Set up output file
		imroot (imobj)
		newfile = imroot.root // ".ms.fits"
		newpath = ddir // newfile
		if (access (newpath))
		    delete (newpath)

		gettime ()
		history = gettime.date // " " // gettime.time

# Fit combined sky spectrum to object+sky and remove it
		if (skymethod == "skyfit") {
		    hskyfit1 (imobj, skycomb, newfile)
		    history = history // " sky fit and removed"
		    }

# Scale combined sky spectrum to object+sky and subtract it
		else {
		    hskysub1 (imobj, skycomb, newfile)
		    history = history // " sky scaled and subtracted"
		    }

# Write HSKYPROC parameters to the image header
		hedit (newpath, "HSKYPROC",history, add+, verify-)
		hedit (newpath, "SKYMETHD",skymethod, add+, verify-)
		hedit (newpath, "NSKYMAX",nskies, add+, verify-)
		hedit (newpath, "SKYDIST",fiberdist, add+, verify-)
		hedit (newpath, "NSKYFIB",hskymatch.nsky, add+, verify-)
		filename (skycomb)
		hedit (newpath, "SKYFILE",filename.filename, add+, verify-)
		printf ("%s	%d	%s\n",imobj,hskymatch.nsky,filename.filename, >> objskydb)
		}
	    else  {
		print ("HSKYFIND: No skies found for ", imobj)
		printf ("%s	0	none\n",imobj, >> objskydb)
		}
	    }

	return

end
# Sep  6 2007	New task based on Bill Wyatt's skyproc ksh script
# Sep 12 2007	Write out db file with sky aperture list
# Sep 13 2007	Use fixdir so destdir doe not need trailing /
