# File hectospec/Scripts/hskylist.cl
# By Doug Mink, after Bill Wyatt's skyproc.ksh
# August 2, 2007

procedure hskylist (spectra)

string spectra = "\*.fits"      {prompt="Files to process"}

string destdir = "skysub/"      {prompt="Output dir for sky-subtracted files"}
bool skysame = yes      {prompt="Use skys only from same chip"}
string skydb = ""               {prompt="Sky filenames and info"}
string skydb1 = ""      {prompt="Chip 1 sky filenames and info"}
string skydb2 = ""      {prompt="Chip 2 sky filenames and info"}
string objdb = ""               {prompt="Object filenames and info"}
string objdb1 = ""      {prompt="Chip 1 object filenames and info"}
string objdb2 = ""      {prompt="Chip 2 object filenames and info"}
bool deltemp = yes      {prompt="Delete temporary files (yes or no"}
bool verbose = yes      {prompt="Log progress (yes or no)"}
bool debug = no         {prompt="Print debugging information (yes or no)"}

begin
    bool vb, dt
    real fiber_x, fiber_y, fiber_radius, fiberlim
    file imf
    int aperture, chipnum, fiber
    int nsky, nsky1, nsky2
    string args, imagetyp, object

    dt = deltemp
    vb = verbose

# limit of non-parked fibers
    fiberlim = 306.0

# Make sure output directory is there
    if (!access (destdir)) {
        mkdir (destdir)
        }

    if (vb) {
        print ("HSKYLIST FiberRadiusLimit = ", fiberlim)
        if (skysame)
            print ("HSKYLIST:  Skys on same chip")
        else
            print ("HSKYLIST:  Skys on either chip")
        print ("HSKYLIST: DestDir = ", destdir)
        }

#  Make temporary list of input spectrum files
    tchar = substr (spectra,1,1)
    if (tchar == '@') {
        templist = substr (spectra,2,100)
        }
    else {
        templist = skydir // "/" // mktemp ("skyproc")
        files (spectra, >templist)
        }

# Generate master table of keywords
    tempkey = skydir // "/" // mktemp ("skykey")
    list = templist
    while (fscan (list, imf) != EOF) {
        args = imf // " -au imagetyp object aperture fiber fiber_x fiber_y"
        gethead (args, >>tempkey)
        }
#  Make temporary list(s) of sky/background spectrum files
    if (samesky) {
        skydb1 = skydir // "/" // mktemp ("skychip1")
        skydb2 = skydir // "/" // mktemp ("skychip2")
        if (vb)
            print ("SKYLIST: Writing skies to ",skydb1," and ",skydb2)
        }
    else {
        skydb = skydir // "/" // mktemp ("skyall")
        if (vb)
            print ("SKYLIST: Writing skies to ",skydb)
        }

#  Make temporary list(s) of object+ sky/background spectrum files
    if (samesky) {
        objdb1 = skydir // "/" // mktemp ("objchip1")
        objdb2 = skydir // "/" // mktemp ("objchip2")
        if (vb)
            print ("SKYLIST: Writing objects to ",objdb1," and ",objdb2)
        }
    else {
        objdb = skydir // "/" // mktemp ("objall")
        if (vb)
            print ("SKYLIST: Writing objects to ",objdb)
        }

# Edit master keyword table
    nsky = 0
    nsky1 = 0
    nsky2 = 0
    nobj = 0
    list = tempkey
    while (fscan (list, imf, imagetype, object, aperture, fiber, fiber_x, fiber_y) != EOF) {
        if (object == "-1") {
            object = "UNUSED"
            imagetyp = "UNUSED"
            }
        else if (object == "-2") {
            object = "BROKEN"
            imagetyp = "BROKEN"
            }
        else if (object == "0") {
            object = "SKY"
            }
        fiber_radius = sqrt (fiber_x * fiber_x + fiber_y * fiber_y)
        if (skysame) {
            if (aperture <= 150) {
                chipnum = 1
                }
            else {
                chipnum = 2
                }
            }
        else {
            chipnum = 0
            }

# Pull all object (including sky) files into a master list
        if ((imagetyp == "object" || imagetyp == "OBJECT)) &&
            object != "UNUSED" && object != "BROKEN" &&
            object != "PARKED && fiber_radius <= fiberlim) {
            if (skysame) {
                if (aperture < 151) {
                    printf ("%s %3d     %3d     %f      %f\n",
                    imf,aperture,fiber,fiber_x,fiber_y,>>objdb1)
                    }
                else {
                    printf ("%s %3d     %3d     %f      %f\n",
                    imf,aperture,fiber,fiber_x,fiber_y,>>objdb2)
                    }
            else {
                printf ("%s     %3d     %3d     %f      %f\n",
                    imf,aperture,fiber,fiber_x,fiber_y,>>objdb)
                }
            nobj = nobj + 1
            }

# create a sky file db
        if ((imagetyp == "OBJECT" && object == "SKY" &&
            fiber_radius <= fiberlim) {
            if (skysame) {
                if (chipnum == 1) {
                    printf ("%s %3d     %3d     %f      %f\n",
                    imf,aperture,fiber,fiber_x,fiber_y,>>skydb1)
                    nsky1 = nsky1 + 1
                    }
                else {
                    printf ("%s %3d     %3d     %f      %f\n",
                    imf,aperture,fiber,fiber_x,fiber_y,>>skydb2)
                    nsky2 = nsky2 + 1
                    }
                }
            else {
                printf ("%s     %3d     %3d     %f      %f\n",
                    imf,aperture,fiber,fiber_x,fiber_y,>>skydb)
                }
            nsky = nsky + 1
            }
        }

# Return if no sky spectra to subtract
        if (skysame)
           nsky = nsky1 + nsky2
        if (nsky <= 0) {
            print ("HSKYLIST: No sky spectra in this directory.")
            exit -1
            }
        }
end
