# HSMIX - Combine multispec sky-subtracted spectra for same field with
#         different pointings
# March 16, 2007
# By Doug Mink, SAO Telescope Data Center

procedure hsmix (indirs, outdir)

string	indirs=""	{prompt="Directories with spectra to combine"}
file	outdir=""	{prompt="Output directory name"}

string	combine = "sum"	{prompt="Combine images (average|median|sum)"}
string	reject = "none"	{prompt="Reject method (none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip)"}
string	scale = "none"	{prompt="Image scaling"}
string	sample = ""	{prompt="Wavelength sample regions for statistics"}
bool	deltemp = no	{prompt="Delete intermediate list files (y or n)"}
bool	verbose = yes	{prompt="Print process progress (y or n)"}
bool	debug = no	{prompt="Print information for debugging (y or n)"}

begin

string	imagelist, history, keyword, arg, dirpaths, imtemp, pathtemp
file	templist, tempdirs, od, catfile
file	imageout, im1, im2, dir1, dir2, images, imp1, imp2
bool	vb, db, samefile
int	i

	vb = verbose
	db = debug
	od = outdir

	if (!defpac ("onedspec")) {
	    onedspec
	    }

# Initialize scombine
	scombine.noutput = ""
	scombine.logfile = "STDOUT"
	scombine.apertures = "1-3"
	scombine.group = "apertures"
	scombine.combine = combine
	scombine.reject = reject
	scombine.first = yes
	scombine.w1 = INDEF
	scombine.w2 = INDEF
	scombine.dw = INDEF
	scombine.nw = INDEF
	scombine.log = no
	scombine.scale = scale
	scombine.zero = "none"
	scombine.weight = "none"
	scombine.sample = sample
	scombine.lthreshold = INDEF
	scombine.hthreshold = INDEF
	scombine.nlow = 1
	scombine.nhigh = 1
	scombine.nkeep = 1
	scombine.mclip = no
	scombine.lsigma = 3.
	scombine.hsigma = 3.
	scombine.rdnoise = "2.7"
	scombine.gain = "1."
	scombine.snoise = "0."
	scombine.sigscale = 0.1
	scombine.pclip = -0.5
	scombine.grow = 0
	scombine.blank = 0.
	scombine.mode = "ql"

#  Make temporary list of input directories
	tempdirs = mktemp ("hsmixdirs")
	files (indirs, >tempdirs)
	list = tempdirs

# Read first directory name
	i = fscan (list,dir1)

	if (od == "")
	    od = "mixed"
	if (!access (od)) {
	    mkdir (od)
	    }

# Read second directory name
	i = fscan (list, dir2)

	templist = mktemp ("hsmixlist1")
	dirpaths = dir1 // "/*.ms.fits"
	if (vb) {
	    print ("Listing files matching ",dirpaths)
	    }
	files (dirpaths, >templist)
	list = templist
	while (fscan (list,imp1) != EOF) {
	    if (vb) {
		print ("Path ",imp1)
		}
	    filename (imp1)
	    im1 = filename.filename
	    catfile = substr (im1,5,100)
	    imtemp = "/*" // catfile
	    pathtemp = dir2 // imtemp
	    if (vb) {
		print ("File ",im1," -> Template ",pathtemp)
		}
	    dir (pathtemp, maxch=256) | scan (im2)
	    imageout = od // "/" // catfile
	    gettime ()
	    if (im2 != "no") {
		imp2 = dir2 // "/" // im2
		images = imp1 // "," // imp2
		scombine (images, imageout)
		if (vb) {
		    print ("HSMIX: 2 images combined into ",imageout)
		    }
		history = gettime.date // " " // gettime.time // " 2 images scombined"
		}
	    else {
		images = dir1 // "/" // im1
		imcopy (images, imageout)
		history = gettime.date//" "//gettime.time//" "//imp1//" copied"
		if (vb) {
		    print ("HSMIX: ",imp1," copied to ",imageout)
		    }
		}
	    hedit (imageout, "HSMIX", history, add+, verify-)
	    }

# Check for files only appearing in the second directory
	list = tempdirs

# Read first directory name
	i = fscan (list,dir1)

# Read second directory name
	i = fscan (list, dir2)

	templist = mktemp ("hsmixlist2")
	dirpaths = dir2 // "/*.ms.fits"
	files (dirpaths, >templist)
	list = templist
	while (fscan (list,imp1) != EOF) {
	    filename (imp1)
	    im1 = filename.filename
	    catfile = substr (im1,5,100)
	    imtemp = "/*" // catfile
	    pathtemp = dir1 // imtemp
	    dir (pathtemp, maxch=256) | scan (im2)
	    imageout = od // "/" // catfile
	    gettime ()
	    if (im2 == "no") {
		imcopy (imp1, imageout)
		history = gettime.date//" "//gettime.time//" "//imp1//" copied"
		if (vb) {
		    print ("HSMIX: ",imp1," copied to ",imageout)
		    }
		}
	    hedit (imageout, "HSMIX", history, add+, verify-)
	    }

	if (deltemp) {
	    delete (templist)
	    delete (tempdirs)
	    }
end

# Mar 16 2007	New program
