# TEXTRACT -- Extract TRES image into two echelle files
# By Doug Mink, SAO Telescope Data Center
# September 13, 2007

procedure textract (images)

file	images=""	{prompt="Input image(s) to extract"}

file	aprefsci=""	{prompt="Science aperture reference spectrum"}
file	aprefsky=""	{prompt="Sky aperture reference spectrum"}
file	apdir=""	{prompt="Directory for reference spectra"}
string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
string	gain="1"	{prompt="Photon gain (photons/data number)"}
real    npeaks=0.2	{prompt="Select brightest peaks"}
bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=yes	{prompt="Print debugging information (y or n)"}

begin
	bool vb, db
	file imin, imsky, imsci, imroot, templist, listfile
	file imrefsci, imrefsky
	int lroot
	string history, ims, tchar

	ims = images
	imrefsky = apdir // aprefsky
	imrefsci = apdir // aprefsci
	vb = verbose
	db = debug

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("tempexlist")
        if (tchar == '@') {
	    listfile = substr (ims,2,100)
	    copy (listfile, templist)
	    }
	else {
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make output file name
	    smatch (imin, ".ec")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    }
		else {
		    imroot = imin
		    }
		}
	    imin = imroot // ".fits"
	    imsky = imroot // "_sky.ec.fits"
	    imsci = imroot // "_sci.ec.fits"

# Make sure this is not already an extracted image
	    imgetp.default = "0"
	    imgetp (imin, "TEXTRACT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TEXTRACT: ", imin, " is already an extracted image")
		    print ("TEXTRACT = ", imgetp.value)
		    }
		next
		}
	    if (db) {
		print ("TEXTRACT: ", imin, " -> ", imsci," and ",imsky)
		}

# Set up apsum parameters
	    apsum.apertures = "1-52"
	    apsum.format = "echelle"
	    apsum.profiles = ""
	    apsum.interactive = no
	    apsum.find = no
	    apsum.recenter = no
	    apsum.resize = no
	    apsum.edit = no
	    apsum.trace = no
	    apsum.fittrace = no
	    apsum.extract = yes
	    apsum.extras = no
	    apsum.review = yes
	    apsum.line = INDEF
	    apsum.nsum = 10
	    apsum.background = "none"
	    apsum.weights = "none"
	    apsum.pfit = "fit1d"
	    apsum.clean = no
	    apsum.skybox = 1
	    apsum.saturation = INDEF
	    apsum.readnoise = "0."
	    apsum.gain = "1."
	    apsum.lsigma = 4.
	    apsum.usigma = 4.
	    apsum.nsubaps = 1

# Extract spectra to multispec file using dohspec
	    apsum.references = imrefsci
	    apsum (imin, output=imsci)
	    apsum.references = imrefsky
	    apsum (imin, output=imsky)

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " extracted from " // imin
	    hedit.show = vb
	    hedit (imsci, "TEXTRACT", history, add+, verify-)
	    hedit (imsky, "TEXTRACT", history, add+, verify-)
	    hedit (imsci, "TEXAPREF", imrefsci, add+, verify-)
	    hedit (imsky, "TEXAPREF", imrefsky, add+, verify-)
	    }

	if (deltemp)
	    delete (templist)

end

# Sep 13 2007	New task based on hectospec.hextract
