# HEXTRACT -- Extract  Hectospec image into multispec file
# By Doug Mink, SAO Telescope Data Center
# March 21, 2005
# WFW 2005-03-29

procedure hextract (images)

file	images=""	{prompt="Input image(s) to extract"}
file	mapfile=""	{prompt="Fiber to sky map file"}

file	mapdir=""	{prompt="Directory for mapfiles"}
file	apref=""	{prompt="Aperture reference spectrum"}
int	fibers=300	{prompt="Number of fibers"}
real	width=8.	{prompt="Width of profiles (pixels)"}
real	minsep=5.	{prompt="Minimum separation between fibers (pixels)"}
real	maxsep=100. 	{prompt="Maximum separation between fibers (pixels)"}
string	readnoise="1"	{prompt="Read out noise sigma (photons)"}
string	gain="1"	{prompt="Photon gain (photons/data number)"}
bool	resize=no	{prompt="Resize object apertures (y or n)"}
bool	recenter=no	{prompt="Recenter object apertures (y or n)"}
bool    shift=yes	{prompt="Use average shift instead of recentering (y or n)"}
real    npeaks=0.2	{prompt="Select brightest peaks"}
bool	deltemp=yes	{prompt="Delete temporary file (y or n)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=yes	{prompt="Print extra information for debugging (y or n)"}

begin
	bool vb, db
	file imin, imout, imroot, imref, templist, mapd, map
	int lroot
	string history, ims, tchar

	ims = images
	imref = apref
	vb = verbose
        db = debug

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("mergelist")
	    files (ims, >templist)
	    }
	list = templist
	while (fscan (list,imin) != EOF) {

# Make sure this is not already an extracted image
	    imgetp.default = "0"
	    imgetp (imin, "HEXTRACT")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("HEXTRACT: ", imin, " is already an extracted image")
		    print ("HEXTRACT = ", imgetp.value)
		    }
		next
		}

# Make output file name
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imout = substr (imin, 1, lroot) // ".ms.fits"
		}
	    else {
		imout = imin // ".ms.fits"
		imin = imin // ".fits"
		}

# Look for map file name in header
	    imgetp.default = "0"
	    imgetp (imin, "MAPFILE")
	    if (imgetp.value != "0") {
		map = imgetp.value
		}
	    else if (mapfile != "") {
		map = mapfile
		}
	    else {
		smatch (imin, ".fits")
		if (smatch.found) {
		    map = substr (imin,1,smatch.start-1) // "_map"
		    }
		else {
		    map = imin // "_map"
		    }
		}
	    map = mapdir // map
	    if (vb) {
		print ("HEXTRACT: ", imin," -> ",imout," using ", imref," and ",map)
		}

# Remove excess space from map file so its lines fit on header lines
	    if (map != "") {
		hsmapfix (map)
		}

# Set up dohspec parameters
	    dohspec.resize = resize
	    dohspec.apidtable = map
	    dohspec.apref = imref
	    dohspec.flat = ""
	    dohspec.throughput = ""
	    dohspec.arcs1 = ""
	    dohspec.arcs2 = ""
	    dohspec.arcreplace = ""
	    dohspec.arctable = ""
	    dohspec.readnoise = readnoise
	    dohspec.gain = gain
	    dohspec.datamax = INDEF
	    dohspec.fibers = fibers
	    dohspec.width = width
	    dohspec.minsep = minsep
	    dohspec.maxsep = maxsep
	    dohspec.crval = "INDEF"
	    dohspec.cdelt = "INDEF"
	    dohspec.objaps = ""
	    dohspec.skyaps = ""
	    dohspec.arcaps = ""
	    dohspec.recenter = recenter
	    dohspec.objbeams = "1"
	    dohspec.skybeams = "0"
	    dohspec.arcbeams = ""
	    dohspec.scattered = no
	    dohspec.fitflat = no
	    dohspec.clean = no
	    dohspec.dispcor = no
	    dohspec.savearcs = no
	    dohspec.skyalign = no
	    dohspec.skysubtract = no
	    dohspec.skyedit = no
	    dohspec.saveskys = no
            if(vb || db) {
       	      dohspec.splot = yes
            } else {
       	      dohspec.splot = no
            }
	    dohspec.redo = no
	    dohspec.update = no
            dohspec.batch = no
	    dohspec.listonly = no
	    dohspec.shift = shift
	    dohspec.npeaks = npeaks
	    dohspec.params = ""

# Extract spectra to multispec file using dohspec
	    dohspec (imin)

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " extracted from " // imin
	    hedit.show = vb
	    hedit (imout, "HEXTRACT", history, add+, verify-)
	    hedit (imout, "HEXAPREF", imref, add+, verify-)
	    hedit (imout, "HEXAPMAP", map, add+, verify-)
	    }
	if (deltemp && tchar != '@') {
	    delete (templist)
	    }

end

# Jun 15 2004	Add recenter, shift, npeaks, parameters for dohspec and proc
# Jun 16 2004	Add resize parameter for dohspec and apscript
# Sep  2 2004	Add HEXTRACT keyword when processed
# Oct 18 2004	Print HEXTRACT keyword if already processed

# Jan 12 2005	Set more dohspec parameters from hextract parameters
# Mar 21 2005	Image map name can be made from input file name .fits -> _map
# Mar 21 2005	Do not change deltemp if reading from file list
# Mar 29 2005   WFW - turn off splot at end unless verbose or debug
